import numpy as np
np.seterr(divide='ignore', invalid='ignore')

from FourierDescriptor import *
from PyQt4.QtGui import *
from PyQt4.QtCore import *

class FourierCanvas(QWidget):
	Action_AddPoints = 1

	def __init__(self, parent = None):
		super(FourierCanvas, self).__init__(parent)
		self.model = None
		self.action = None
		self.setStyleSheet("FourierCanvas { background-color : white; }");

	def paintEvent(self, event):
		painter = QPainter(self)
		option = QStyleOption()
		option.initFrom(self)
		self.style().drawPrimitive(QStyle.PE_Widget, option, painter, self)

		n = len(self.model.pointSet)
		if n == 0:
			# Nothing to draw.
			return

		def drawPoints(pointSet, color = QColor(0, 0, 0), radius = 5):
			painter.setPen(QPen(QBrush(color), radius, cap = Qt.RoundCap))
			qPolygon = QPolygonF(
				[QPointF(point.real, point.imag) for point in pointSet]
			)
			painter.drawPoints(qPolygon)

		painter.setRenderHint(QPainter.Antialiasing);
		painter.setRenderHint(QPainter.HighQualityAntialiasing);

		# Draw the actual points.
		drawPoints(self.model.noisySet, QColor(255, 0, 0))

		# Draw arc-length parametrization
		drawPoints(self.model.arcSet, QColor(0, 255, 0, 128), radius = 10)

		if n <= 1:
			# Cannot draw a curve, since there is only one point.
			return

		# Draw the moving markers.
		t = 2 * (float(self.model.step) / self.model.steps) 
		if self.model.useArcLength:
			t *= (self.model.length / n) / self.model.arcLength

		markers = int(np.ceil(self.model.length / 50).item()) + 1

		sSet = shiftFourierDescriptor(self.model.fSet, t)
		ySet = inverseFourierDescriptor(sSet, points = markers, terms = self.model.terms)
		
		drawPoints(ySet, QColor(0, 0, 255, 128), radius = 10)

		# Draw the continuous curve.
		ySet = inverseFourierDescriptor(sSet, points = self.model.pathDetail * n, terms = self.model.terms)

		painter.setPen(QPen(QBrush(QColor(0, 0, 0)), 1, cap = Qt.RoundCap))
		qPolygon = QPolygonF(
			[QPointF(point.real.item(), point.imag.item()) for point in ySet]
		)
		painter.drawPolygon(qPolygon)

	def mousePressEvent(self, event):
		if event.button() == Qt.LeftButton:
			self.action = self.Action_AddPoints

	def mouseReleaseEvent(self, event):
		if event.button() == Qt.LeftButton:
			if self.action is self.Action_AddPoints:
				self.action = None
		if event.button() == Qt.RightButton:
			self.model.pointSet = []
			self.model.noiseSet = []
			self.model.recompute()

	def mouseMoveEvent(self, event):
		if self.action is self.Action_AddPoints:
			position = event.pos()
			self.model.pointSet.append(complex(position.x(), position.y()))
			self.model.noiseSet.append(np.random.randn() + 1j * np.random.randn())
			self.model.recompute()
