import numpy as np

from scipy import special
from scipy import optimize

class Kaiser_Filter(object):
	def __init__(self, beta):
		self.beta = beta

	def __call__(self, x):
		y = np.zeros(x.size)
		if self.beta == 0:
			y[np.abs(x) <= 1] = 1
		else:
			y[np.abs(x) <= 1] = np.i0(np.pi * self.beta * np.sqrt(1 - x[np.abs(x) <= 1]**2)) / np.i0(np.pi * self.beta)
		return y

def sidi(x, m):
	z = special.gamma(m / 2.0) * special.gamma((m + 1) / 2.0 - x) * special.gamma((m + 1) / 2.0 + x)
	y = 2**(1.0 - m) * np.sqrt(np.pi) * special.gamma(m) * special.gamma((m + 1) / 2.0) / z
	return y

class Sidi_Filter(object):
	def __init__(self, m):
		self.m = m
		# The first non-negative root of sidi_m is given by 0.5 (m + 1).
		self.firstRoot = (m + 1) / 2.0

	def __call__(self, x):
		return sidi(x * self.firstRoot, self.m)

def korobev(x, m):
	z = np.pi**(m - 1) * special.gamma(m) * np.abs(2 * x)**(m - 0.5)
	y = special.gamma(2 * m) * special.jv(m - 0.5, np.pi * np.abs(x)) / z
	if type(x) is np.ndarray:
		y[np.where(np.isnan(y))] = 1
	else:
		if np.isnan(y):
			y = 1
	return y

class Korobev_Filter(object):
	def __init__(self, m):
		self.m = m
		# The first non-negative root of korobev_m has to
		# be found numerically (or at least I don't know
		# a closed-form solution).
		self.firstRoot = optimize.newton(lambda x : korobev(x, m), 0.5)

	def __call__(self, x):
		return korobev(x * self.firstRoot, self.m)

class Sinc_Filter(object):
	def __call__(self, x):
		return np.sinc(x)

class Cut_Filter(object):
	def __call__(self, x):
		return 1
